package display;

import java.util.*;
import java.awt.geom.*;

/**
Klasse zur Bereitstellung eines Bruchs.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class DispFract extends DispObject {

 /**
 Aufbau eines MathFract-Objekts.
 @param mf Instanz eines MathFract-Objekts, welches (neu) aufzubauen ist.
 @param num Das Zähler-Objekt.
 @param den Das Nenner-Objekt.
 */
 static public void build( DispObject mf, DispObject num, DispObject den) {
  // Skalieren des Bruchstrichs
 DispObject mc = new DispObject();
  DispObject.characterData( mc, '-');
  DispObject.compress( mc);
 double max = Math.max( num.bnd.getWidth(), den.bnd.getWidth());
  DispObject.scale( mc, (max+mc.bnd.getWidth()/2), DispObject.HORIZONTAL);
  // Liste der Objekte aufbauen
  DispObject.append( mf, num);
  DispObject.append( mf, mc);
  DispObject.append( mf, den);
  // Alignment von numerator, line, denumerator
 Rectangle2D[] tupel = new Rectangle2D[3];
 Iterator      iter  = mf.objects.iterator();
  for (int i = 0; i < tupel.length; i++)
       tupel[i] = ((DispObject)iter.next()).bnd;
  tupel = Align.alignCol( tupel, Align.CENTER, 0);
  // Bounds ermitteln
  DispObject.bounds( mf, tupel);
  // Referenz für Baseline festlegen
 DispObject ref = new DispObject();
  DispObject.characterData( ref, '-');
  // Baseline festlegen
  mf.bse = mc.bnd.getY()+Math.abs( ref.shp.getBounds2D().getY()-ref.bse);
 }

 /**
 Spezifische Ausgabe für Objekte der MathString-Klasse.
 @param deep Tiefe der Einrückung. Dient der strukturierten Ausgabe.
 @param obj Objekt für textueller Ausgabe.
                       Es sollte sich unbedingt um ein MathString-Objekt handeln.
 */
 static public void printSpecial( int deep, DispObject obj) {
 Iterator   iter = obj.objects.iterator();
  DispObject.print( deep+1, (DispObject)iter.next()); // Zähler
  DispObject.out.print( ",");
  iter.next();                                        // "Bruchstrich" überspringen
  DispObject.print( deep+1, (DispObject)iter.next()); // Zähler
 }

 /**
 Creator eines MathFract-Objekts. Dieser "leere" Creator dient einer weiteren
 Flexibilisierung durch Trennung von Instanzierung und dem eigentlichen
 Aufbau des Objekts mit der Methode "build".
@see build.
 */
 public DispFract() {
 }

 /**
 Creator eines MathFract-Objekts.
 @param num Zähler; kann jedes von MathObject abgeleitete Objekt sein.
 @param den Nenner; kann jedes von MathObject abgeleitete Objekt sein.
 */
 public DispFract( DispObject num, DispObject den) {
  build( this, num, den);
 }
}
