package display;

import java.util.*;
import java.awt.geom.*;

/**
Klasse zur Bereitstellung von Ausdrücken mit Skripten.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class DispMultiScript extends DispObject {

 /**
 Aufbau eines MathSubsup-Objekts.
 @param obj Instanz eines MathSubsup-Objekts, welches (neu) aufzubauen ist.
 @param base Basis-Objekt, das ein kombiniertes Sub- Superscript erhält.
 @param sub Subscript-Objekt des Basis-Objekts.
 @param sup Superscript-Objekt des Basis-Objekts.
 */
 static public void build( DispObject obj, DispObject base, DispObject rsub, DispObject rsup, DispObject lsub, DispObject lsup) {
  // interne Liste aufbauen
  DispObject.append( obj, base);
  DispObject.append( obj, rsub);
  DispObject.append( obj, rsup);
  DispObject.append( obj, lsub);
  DispObject.append( obj, lsup);
  // Alignment der Scripts
 double dx = 0;
 double dy = 0;
  if (rsub != null)
     {
      DispObject.resize( rsub, DispObject.subScale);
      dx = base.bnd.getX() + base.bnd.getWidth() + DispObject.defaultCharSpace;
      dy = base.bnd.getHeight() * (1 - DispObject.subScript);
      DispObject.relPos( rsub, dx, dy);
     }
  if (rsup != null)
     {
      DispObject.resize( rsup, DispObject.supScale);
      dx = base.bnd.getX() + base.bnd.getWidth() + DispObject.defaultCharSpace;
      dy = base.bnd.getHeight() * DispObject.supScript - rsup.bnd.getHeight();
      DispObject.relPos( rsup, dx, dy);
     }
  if (lsub != null)
     {
      DispObject.resize( lsub, DispObject.subScale);
      dx = base.bnd.getX() - lsub.bnd.getWidth() - DispObject.defaultCharSpace;
      dy = base.bnd.getHeight() * (1 - DispObject.subScript);
      DispObject.relPos( lsub, dx, dy);
     }
  if (lsup != null)
     {
      DispObject.resize( lsup, DispObject.supScale);
      dx = base.bnd.getX() - lsup.bnd.getWidth() - DispObject.defaultCharSpace;
      dy = base.bnd.getHeight() * DispObject.supScript - lsup.bnd.getHeight();
      DispObject.relPos( lsup, dx, dy);
     }
  // Bounds ermitteln
  obj.bnd = new Rectangle2D.Double();
 Iterator iter = obj.objects.iterator();
  while (iter.hasNext()) 
	    {
	    DispObject elm = (DispObject)iter.next();
	     if (elm != null) Rectangle2D.union( obj.bnd, elm.bnd, obj.bnd);
	    }
  // Objekt auf Position 0, 0 bringen
  DispObject.relPos( obj, -obj.bnd.getX(), -obj.bnd.getY());
  // Baseline endgültig festlegen
  DispObject elm = (DispObject)obj.objects.getFirst();
  obj.bse = elm.bse + elm.bnd.getY();
 }

 /**
 Spezifische Ausgabe für Objekte der MathString-Klasse.
 @param deep Tiefe der Einrückung. Dient der strukturierten Ausgabe.
 @param obj Objekt für textueller Ausgabe.
 */
 static public void printSpecial( int deep, DispObject obj) {
 Iterator iter = obj.objects.iterator();

  iter = obj.objects.iterator();
  DispObject.print( deep+1, (DispObject)iter.next());
  DispObject.out.print( ",");

  while (iter.hasNext())
        {
	    DispObject elm = (DispObject)iter.next();
	     if   (elm != null)
              {
	    	   DispObject.print( deep+1, elm);
	    	   if (iter.hasNext()) DispObject.out.print( ",");
              }
	     else {
               DispObject.out.println();
               for (int s=0; s<deep+1; s++) DispObject.out.print( " ");
               DispObject.out.print( "null");
	    	   if (iter.hasNext()) DispObject.out.print( ",");
              }
        }
 }

 /**
 Creator eines MathSubsup-Objekts. Dieser "leere" Creator dient einer weiteren
 Flexibilisierung durch Trennung von Instanzierung und dem eigentlichen
 Aufbau des Objekts mit der Methode "build".
@see build.
 */
 public DispMultiScript() {
 }

 /**
 Creator eines MathSubsup-Objekts.
 @param base Basis-Objekt, das ein tiefergestelltes Sub-Objekt erhält (Index).
 @param base Basis-Objekt, das ein kombiniertes Sub- Superscript erhält.
 @param sub Subscript-Objekt des Basis-Objekts.
 @param sup Superscript-Objekt des Basis-Objekts.
 */
 public DispMultiScript( DispObject base, DispObject rsub, DispObject rsup, DispObject lsub, DispObject lsup) {
  build( this, base, rsub, rsup, lsub, lsup);
 }
}
