package display;

import java.util.*;
import java.awt.geom.*;

/**
Klasse zur horizontalen Zusammenfassung von beliebigen DispObjects.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class DispRow extends DispObject {

 /**
 Aufbau eines MathRow-Objekts.
 @param mr Instanz eines MathRow-Objekts, welches (neu) aufzubauen ist.
 @param list Liste mit Objekten der Zeile.
 @param fnt Der zu verwendende Zeichensatz.
 @param frc Der Darstellungskontext für die Zeichen (ist abhängig vom Ausgabemedium).
 */
 static public void build( DispObject mr, LinkedList list, double space) {
  // Liste der Elemente intern führen
 Iterator iter = list.iterator();
  while (iter.hasNext()) DispObject.append( mr, (DispObject)iter.next());
  // Alignment aller Elemente
 Rectangle2D[] tupel = new Rectangle2D[mr.objects.size()];
  iter = mr.objects.iterator();
  for (int i = 0; i < tupel.length; i++)
       tupel[i] = ((DispObject)iter.next()).bnd;
  tupel = Align.alignRow( tupel, Align.TOP, space); // HIER noch den Abstand ermitteln!!!!!
  // vorläufige Baseline für Aufbau festlegen (kann sich ändern)
  mr.bse = ((DispObject)mr.objects.getFirst()).bse;
  // Bounds ermitteln
  mr.bnd = new Rectangle2D.Double();
  iter = mr.objects.iterator();
 DispObject elm;
 double     dx;
 double     dy;
  for (int i = 0; i < tupel.length; i++)
       {
        elm = (DispObject)iter.next();
        dx = tupel[i].getX() - elm.bnd.getX();
        dy = tupel[i].getY() - elm.bnd.getY();
        dy += mr.bse - elm.bse; // Bezug auf vorläufige Baseline
        DispObject.relPos( elm, dx, dy);
        Rectangle2D.union( mr.bnd, elm.bnd, mr.bnd);
       }
  // Objekt auf Position 0, 0 bringen
  DispObject.relPos( mr, 0, -mr.bnd.getY());
  // Baseline endgültig festlegen
  elm = (DispObject)mr.objects.getFirst();
  mr.bse = elm.bnd.getY() + elm.bse;
 }

 /**
 Spezifische Ausgabe für Objekte der MathRow-Klasse.
 @param deep Tiefe der Einrückung. Dient der strukturierten Ausgabe.
 @param obj Objekt für textueller Ausgabe.
 */
 static public void printSpecial( int deep, DispObject obj) {
 Iterator iter = obj.objects.iterator();

  while (iter.hasNext())
         {
          DispObject.print( deep+1, (DispObject)iter.next());
          if (iter.hasNext()) DispObject.out.print( ",");
         }
 }

 /**
 Creator eines MathRow-Objekts. Dieser "leere" Creator dient einer weiteren
 Flexibilisierung durch Trennung von Instanzierung und dem eigentlichen
 Aufbau des Objekts mit der Methode "build".
@see build.
 */
 public DispRow() {
 }

 /**
 Creator eines MathRow-Objekts.
 @param list Liste mit Objekten; kann jedes von MathObject abgeleitete Objekt sein.
 */
 public DispRow( LinkedList list) {
  build( this, list, DispObject.defaultRowSpace);
 }
 
 /**
 Creator eines MathRow-Objekts.
 @param list Liste mit Objekten; kann jedes von MathObject abgeleitete Objekt sein.
 @param space Horizontaler Abstand zwischen den Objekten. 
 */
 public DispRow( LinkedList list, double space) {
  build( this, list, space);
 }
 
 /**
 Creator eines MathRow-Objekts.
 @param obj Array mit Objekten; kann jedes von MathObject abgeleitete Objekt sein.
 */
 public DispRow( DispObject[] obj) {
 LinkedList list = new LinkedList();
  for (int i=0; i<obj.length; i++)
	  list.add( obj[i]);
  build( this, list, DispObject.defaultRowSpace);
 }

 /**
 Creator eines MathRow-Objekts.
 @param obj Array mit Objekten; kann jedes von MathObject abgeleitete Objekt sein.
 @param space Horizontaler Abstand zwischen den Objekten. 
 */
 public DispRow( DispObject[] obj, double space) {
 LinkedList list = new LinkedList();
  for (int i=0; i<obj.length; i++)
	  list.add( obj[i]);
  build( this, list, space);
 }
}
