package display;

import java.util.*;
import java.awt.geom.*;

/**
Klasse zur Bereistellung der Quadratwurzel aus einem Ausdruck.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class DispSqrt extends DispObject {

 /**
 Aufbau eines MathSqrt-Objekts.
 @param obj Instanz eines MathSqrt-Objekts, welches (neu) aufzubauen ist.
 @param bse Das zu radizierende Objekt.
 */
 static public void build( DispObject obj, DispObject bse) {
 DispObject mc = new DispObject();
  DispObject.characterData( mc, '\u221A'); // Wurzel
  DispObject.compress( mc);

 DispOverline ovl = new DispOverline();
  DispOverline.build( ovl, '|', bse);

  DispObject.scale( mc, bse.bnd.getHeight(), DispObject.VERTICAL);
//  mc.bse = bse.bse;
  DispObject.append( obj, mc);
  DispObject.append( obj, ovl);

 Rectangle2D[] tupel = new Rectangle2D[2];
 Iterator      iter  = obj.objects.iterator();
  for (int i = 0; i < tupel.length; i++)
       tupel[i] = ((DispObject)iter.next()).bnd;
  tupel = Align.alignRow( tupel, Align.TOP, 0);
  // Bounds ermitteln
  DispObject.bounds( obj, tupel);
  // Baseline festlegen
  mc.bse  = ovl.bse;
  obj.bse = ovl.bse;
 }

 /**
 Spezifische Ausgabe für Objekte der MathSqrt-Klasse.
 @param deep Tiefe der Einrückung. Dient der strukturierten Ausgabe.
 @param obj Objekt für textueller Ausgabe.
 */
 static public void printSpecial( int deep, DispObject obj) {
 Iterator iter = obj.objects.iterator();
  iter.next();                             // Wurzelzeichen überspringen.
 DispObject ovl = (DispObject)iter.next(); // im Overline-Object
  iter = ovl.objects.iterator();           // Objektliste
  iter.next();                             // Overline-Zeichen überspringen
  DispObject.print( deep+1, (DispObject)iter.next());
 }

 /**
 Creator eines MathSqrt-Objekts. Dieser "leere" Creator dient einer weiteren
 Flexibilisierung durch Trennung von Instanzierung und dem eigentlichen
 Aufbau des Objekts mit der Methode "build".
@see build.
 */
 public DispSqrt() {
 }

 /**
 Creator eines MathSqrt-Objekts.
 @param bse Das zu radizierende Objekt.
 */
 public DispSqrt( DispObject bse) {
  build( this, bse);
 }
}
