package display;

import java.util.*;
import java.awt.geom.*;

/**
Klasse zum Aufbau von Tabellen.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class DispTab extends DispObject {
/**
Spaltenanzahl der Tabelle
*/
public int cols;

/**
Zeilenanzahl der Tabelle
*/
public int rows;

 /**
 Aufbau eines MathTab-Objekts.
 @param mr Instanz eines MathRow-Objekts, welches (neu) aufzubauen ist.
 @param list Zweidimensionales Array mit den Elementen.
 @param horhow Horizontale Ausrichtung 
 @param rowsp Minimaler Abstand zwischen den ausgerichteten Spalten. 
 @param verhow Vertikale Ausrichtung.
 @param versp Minimaler Abstand zwischen den ausgerichteten Zeilen.
 */
 static public void build( DispObject mr, 
                           DispObject[][] list, 
		                   int        horhow, 
		                   double     rowsp,
		                   int        verhow,
		                   double     versp) {
  // Zeilen- und Spaltenlänge ermitteln
  ((DispTab)mr).cols = list[0].length; // list.size();
  ((DispTab)mr).rows = list.length; // ((MathObject)list.getFirst()).objects.size();
  // Liste der Elemente intern führen
  Rectangle2D[][] table = new Rectangle2D[((DispTab)mr).rows][((DispTab)mr).cols];
  for (int i=0; i<((DispTab)mr).cols; i++)
	  for (int j=0; j<((DispTab)mr).rows; j++)
          {
		   DispObject.append( mr, list[j][i]); // HIER!!!!!
		   table[j][i] = ((DispObject)list[j][i]).bnd;
          }
  table = Align.alignTable( table, horhow, rowsp, verhow, versp);
  // Bounds ermitteln
 Rectangle2D[] tupel = new Rectangle2D[mr.objects.size()]; 
 int k = 0;
  for (int i=0; i<((DispTab)mr).cols; i++)
   for (int j=0; j<((DispTab)mr).rows; j++)
	    tupel[k++] = table[j][i];
  DispObject.bounds( mr, tupel);
  // Baseline festlegen
  DispObject ref = new DispObject();
  DispObject.characterData( ref, '-');
  mr.bse = mr.bnd.getHeight() / 2.0 + Math.abs( ref.shp.getBounds2D().getY()-ref.bse);
 }

 /**
 Spezifische Ausgabe für Objekte dieser Klasse.
 @param deep Tiefe der Einrückung. Dient der strukturierten Ausgabe.
 @param obj Objekt für textueller Ausgabe.
 */
 static public void printSpecial( int deep, DispObject obj) {
 Iterator iter = obj.objects.iterator();

  DispObject.out.print( " [cols="+((DispTab)obj).cols+", rows="+((DispTab)obj).rows+"]");

  while (iter.hasNext())
         {
          DispObject.print( deep+1, (DispObject)iter.next());
          if (iter.hasNext()) DispObject.out.print( ",");
         }
 }

 /**
 Creator eines MathTab-Objekts. Dieser "leere" Creator dient einer weiteren
 Flexibilisierung durch Trennung von Instanzierung und dem eigentlichen
 Aufbau des Objekts mit der Methode "build".
@see build.
 */
 public DispTab() {
 }

 /**
 Creator eines MathTab-Objekts.
 @param list Zweidimensionales Array mit den Elementen.
 @param horhow Horizontale Ausrichtung 
 @param rowsp Minimaler Abstand zwischen den ausgerichteten Spalten. 
 @param verhow Vertikale Ausrichtung.
 @param versp Minimaler Abstand zwischen den ausgerichteten Zeilen.
 */
 public DispTab( DispObject[][] list,
		         int        horhow, 
                 double     rowsp,
                 int        verhow,
                 double     versp) {
  build( this, list, horhow, rowsp, verhow, versp);
 }
}
