
package handlers;

import translators.*; // Translation in grafischen Kontext
import trees.*;	      // eigenes Tree-Pack

/**
Klasse zum Parsen und zur Translation von MathML-Sequenzen.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class MathDocHandler extends org.xml.sax.helpers.DefaultHandler {
/**
Einfacher Tree, der in jeder Methode dieser Klasse benutzt wird.
*/
public Tree tree = new Tree();
/**
Instanz der Translation. Hier nur die grafische Darstellung.
*/
public DocToDisplay type = new DocToDisplay();

 /** 
 Verhalten beim Start eines Tags.
 Der <...>-Token wird in den Tree eingefügt, die folgenden Elemente angehängt.
 */
 public void startElement( String namespaceURI, String sName, String element, org.xml.sax.Attributes attrs) {
  Tree.insert( tree, element);
  tree.append = false; // nächstes Element als Child ansehen
 }

 /** 
 Verhalten beim Ende eines Tags.
 Der </...>-Token wird eine Ebene höher an den Tree angehängt.
 */
 public void endElement( String namespaceURI, String sName, String element) {
  Tree.levelUp( tree); // auf Elterebene
  tree.append = true;  // nächstes Element als Geschwister ansehen
 }

 /** 
 Verhalten beim Inhalt eines Tags.
 Der Inhalt zwischen <...>- und </...>-Tags wird ermittelt und angehängt.
 Leere Tags, also auch solche mit white spaces, werden ignoriert. 
 */
 public void characters( char[] ch, int start, int length) {
 String element = new String( ch, start, length);

  // white spaces entfernen und wenn Nichts bleibt - fertig
  element = element.trim();
  if (element.length() < 1) return;
  Tree.insert( tree, element);
  tree.append = true; // nächstes Element als Geschwister ansehen
 }

 /**
 Einfache Ausgabe des Inhalts eines Nodes.  
 @param node  Auszugebender Node.
 @param level Tiefe der Einrückung.
 */
 public void print( String node, int level) {
  for (int i=0; i < level; i++) System.out.print( " ");
  System.out.println( node);
 }

 /**
 Aufbereitung des geparsten MathML-Trees.
 @param node Node, ab dem die Ausgabe beginnt.
 @param level Anfängliche Tiefe der Einrückung.
 @paran show strukturierte Ausgabe des Trees wenn true.
 */
 public void build( Node node, int level, boolean show) {
  if (node == null) return; // fertig
  // Test, ob Kennung für Ende einer variablen Argumentliste
  if (node.child != null)
	  if (node.child.prev == null)
	     {
		  type.hold( null); // Ende der Argumentliste kennzeichnen
          if (show) print( "null", level);
	     }
  // rekursiver Aufruf für Kindliste
  build( node.child, level+1, show);
  // Auswahl der Aufbereitung in Abhängigkeit des Tokens
 String str = (String)node.object;
  if (show) print( (String)node.object, level);
       if (str.equals( "mi"))         type.strtok( str);
  else if (str.equals( "mn"))         type.strtok( str);
  else if (str.equals( "mo"))         type.strtok( str);
  else if (str.equals( "mtext"))      type.strtok( str); // ohne white spaces
  else if (str.equals( "mrow"))       type.rowtok();
  else if (str.equals( "mfrac"))      type.frctok();
  else if (str.equals( "msqrt"))      type.sqrtok();
  else if (str.equals( "msub"))       type.subtok();
  else if (str.equals( "msup"))       type.suptok();
  else if (str.equals( "msubsup"))    type.sbptok();
  else if (str.equals( "mtd"))        type.mtdtok(); // nur in mrow erlaubt
  else if (str.equals( "mtr"))        type.mtrtok(); // nur in mtable erlaubt
  else if (str.equals( "mtable"))     type.mtbtok();
  else if (str.equals( "munder"))     type.udrtok();
  else if (str.equals( "mover"))      type.ovrtok();
  else if (str.equals( "munderover")) type.uovtok();
  else if (str.equals( "mfenced"))    type.mfctok();
  else if (str.equals( "math"))       type.mthtok();
  else type.hold( str);
  // rekursiver Aufruf für weitere Geschwister
  build( node.next, level, show);
 } 
}
